#!/usr/bin/env python3
# __main__.py

"""
Copyright (C) 2017-2018 [RnD]²

This file is part of cpupower-gui.

cpupower-gui is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

cpupower-gui is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with cpupower-gui.  If not, see <http://www.gnu.org/licenses/>.

Author: Evangelos Rigas <erigas@rnd2.org>
"""

import os
import sys
import dbus
import gi

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

SYS_PATH = "/sys/devices/system/cpu/cpu{}/cpufreq"
FREQ_MIN = "scaling_min_freq"
FREQ_MAX = "scaling_max_freq"
FREQ_MIN_HW = "cpuinfo_min_freq"
FREQ_MAX_HW = "cpuinfo_max_freq"
AVAIL_GOV = "scaling_available_governors"
GOVERNOR = "scaling_governor"

bus = dbus.SystemBus()
session = bus.get_object("org.rnd2.cpupower_gui.helper", "/org/rnd2/cpupower_gui/helper")

helper = dbus.Interface(session, "org.rnd2.cpupower_gui.helper")

def read_settings(cpu):
    """ Reads files from sysfs """
    sys_path = SYS_PATH.format(cpu)
    with open(os.path.join(sys_path, FREQ_MIN), "r") as sys_file:
        freq_min = int(sys_file.readline())

    with open(os.path.join(sys_path, FREQ_MAX), "r") as sys_file:
        freq_max = int(sys_file.readline())

    with open(os.path.join(sys_path, FREQ_MIN_HW), "r") as sys_file:
        freq_minhw = int(sys_file.readline())

    with open(os.path.join(sys_path, FREQ_MAX_HW), "r") as sys_file:
        freq_maxhw = int(sys_file.readline())

    with open(os.path.join(sys_path, AVAIL_GOV), "r") as sys_file:
        govs = sys_file.readline().strip().split(" ")
        governors = {}
        for num, item in enumerate(govs):
            governors[num] = item

    with open(os.path.join(sys_path, GOVERNOR), "r") as sys_file:
        governor = sys_file.readline().strip()

    return freq_min, freq_minhw, freq_max, freq_maxhw, governors, governor

def dialog_response(widget, response_id):
    """ Error message dialog """
    # if the button clicked gives response OK (-5)
    if response_id == Gtk.ResponseType.OK:
        print("OK")
    # if the messagedialog is destroyed (by pressing ESC)
    elif response_id == Gtk.ResponseType.DELETE_EVENT:
        print("dialog closed or cancelled")
    widget.destroy()


class Application(Gtk.Application):
    """ Main Application, inherits from Gtk.Application"""
    ONLINE = '/sys/devices/system/cpu/online'
    ATBOOT = False

    def __init__(self, **kwargs):
        """ Init Gtk.Application"""
        super().__init__(application_id='org.rnd2.cpupower_gui',
                         **kwargs)

        with open(self.ONLINE, "r") as sys_file:
            cpu_count = sys_file.readline().strip().split('-')[-1]
            self.cpu_avail = int(cpu_count) + 1

        self.widgets = {}
        self.builder = Gtk.Builder()
        self.builder.add_from_file("/usr/share/cpupower-gui/ui/cpupower.glade")
        _, self.freq_min_hw, _, self.freq_max_hw, self.governors, self.governor = read_settings(0)
        cpu_store = Gtk.ListStore(int, int)

        for i in range(self.cpu_avail):
            cpu_store.append([i, i])

        self.window = self['window']
        self['cpu_box'].set_model(cpu_store)
        self['cpu_box'].set_active(0)
        self.upd_sliders()
        self.status_icon = self['status']
        self.status_icon.connect("popup-menu", self.right_click_event)
        self.status_icon.connect("activate", self.status_activate)

    def right_click_event(self, icon, button, time):
        self.menu = Gtk.Menu()
        about = Gtk.MenuItem()
        about.set_label("About")
        about.connect("activate", self.on_about_clicked)
        self.menu.append(about)
        quit = Gtk.MenuItem()
        quit.set_label("Quit")
        quit.connect("activate", self.quit)
        self.menu.append(quit)
        self.menu.show_all()
        self.menu.popup(None, None, None, self.status_icon, button, time)

    def status_activate(self, status_icon):
        self.window.deiconify()
        self.window.present()

    def do_activate(self):
        """ Connects signals and start main loop """
        self.builder.connect_signals(self)
        self.window.show()
        Gtk.main()

    def quit(self, *args):
        #helper.quit()
        #self.status_icon.destroy()
        Gtk.main_quit(*args)

    def upd_sliders(self):
        """ Updates the slider widgets by reading the sys files"""
        cpu = self['cpu_box'].get_active()
        freq_min, self.freq_min_hw, freq_max, self.freq_max_hw, self.governors, self.governor = read_settings(cpu)
        gov_store = Gtk.ListStore(str, int)
        for gov in self.governors.items():
            if gov[1] == self.governor:
                gov_id = gov[0]
            gov_store.append([gov[1].capitalize(), gov[0]])

        self['gov_box'].set_model(gov_store)
        self['gov_box'].set_active(gov_id)

        self['adj_min'].set_lower(int(self.freq_min_hw / 1000))
        self['adj_min'].set_upper(int(self.freq_max_hw / 1000))
        self['adj_max'].set_lower(int(self.freq_min_hw / 1000))
        self['adj_max'].set_upper(int(self.freq_max_hw / 1000))
        self['adj_min'].set_value(int(freq_min / 1000))
        self['adj_max'].set_value(int(freq_max / 1000))
        self['apply_btn'].set_sensitive(False)

    def on_quit_clicked(self, *args):
        """ Exit application with cancel button """
        self.quit(*args)

    def on_window_destroy(self, *args):
        """ Exit application with destroy """
        self.quit(*args)

    def on_cpu_changed(self, *args):
        """ Callback for cpu box """
        # pylint: disable=W0612,W0613
        self.upd_sliders()

    def on_toall_state_set(self, _, val):
        """ Enable/Disable cpu_box """
        if val:
            self['cpu_box'].set_sensitive(False)
        else:
            self['cpu_box'].set_sensitive(True)

    def on_atboot_state_set(self, _,val):
        if val:
            self.ATBOOT = True
            print(helper.isauthorized())
        else:
            self.ATBOOT = False

    def on_adj_min_value_changed(self, *args):
        """ Callback for adj_min """
        # pylint: disable=W0612,W0613
        if self['adj_min'].get_value() > self['adj_max'].get_value():
            if self['adj_min'].get_value() + 10 > self.freq_max_hw / 1000:
                self['adj_max'].set_value(self.freq_max_hw / 1000)
            else:
                self['adj_max'].set_value(self['adj_min'].get_value() + 10)
        elif self['adj_max'].get_value() < self['adj_min'].get_value():
            if self['adj_max'].get_value() - 10 < self.freq_min_hw / 1000:
                self['adj_min'].set_value(self.freq_min_hw / 1000)
            else:
                self['adj_min'].set_value(self['adj_max'].get_value() - 10)
        self['apply_btn'].set_sensitive(True)

    def on_adj_max_value_changed(self, *args):
        """ Callback for adj_max """
        # pylint: disable=W0612,W0613
        if self['adj_max'].get_value() < self['adj_min'].get_value():
            self['adj_min'].set_value(self['adj_max'].get_value() - 10)
        self['apply_btn'].set_sensitive(True)

    def on_governor_changed(self, *args):
        """ Change governor and enable apply_btn """
        # pylint: disable=W0612,W0613
        mod = self['gov_box'].get_model()
        text, tid = mod[self['gov_box'].get_active_iter()][:2]
        self.governor = self.governors[tid]
        self['apply_btn'].set_sensitive(True)

    def on_apply_clicked(self, button):
        """ Write changes back to sysfs """
        fmin = int(self['adj_min'].get_value() * 1000)
        fmax = int(self['adj_max'].get_value() * 1000)
        if helper.isauthorized():
            if self['toall'].get_active():
                for i in range(self.cpu_avail):
                    helper.update_cpu_settings(i, fmin, fmax, self.governor)
            else:
                cpu = self['cpu_box'].get_active()
                helper.update_cpu_settings(cpu, fmin, fmax, self.governor)
            button.set_sensitive(False)
        else:
            message = Gtk.MessageDialog(type=Gtk.MessageType.ERROR, buttons=Gtk.ButtonsType.OK)
            message.set_markup("You don't have permissions to update cpu settings!")
            message.show()
            message.connect("response", dialog_response)

    def on_about_clicked(self, button):
        self.show_about_dialog()

    def show_about_dialog(self):
        self['about_dialog'].run()
        self['about_dialog'].hide()

    def on_config_box_changed(self, button):
        self['about_dialog'].hide()

    def __getitem__(self, name):
        """ Convince method that allows widgets to be accessed via self["widget"] """
        if name in self.widgets:
            return self.widgets[name]
        return self.builder.get_object(name)

    def __setitem__(self, name, item):
        """ Convince method that allows widgets to be accessed via self["widget"] """
        self.widgets[name] = item

    def __contains__(self, name):
        """ Returns True if there is such widget """
        if name in self.widgets:
            return True
        return self.builder.get_object(name) is not None


def main():
    """ Main runner"""
    application = Application()

    try:
        ret = application.run()  # sys.argv)
    except SystemExit as sys_exit:
        ret = sys_exit.code
    sys.exit(ret)


if __name__ == '__main__':
    main()


